/* Test pattern for Diamond SpeedStar 24 direct color mode (16M colors)     */
/* The test pattern creates a 640x480 display of expanding circles of       */
/* increasing color with red originating from the upper left corner, green  */
/* from the upper right corner, and blue from the middle of the bottom line.*/
/* Other display adapters with the Tseng ET4000 and direct color modes may  */
/* be able to use this program, or at least with minor modifications.       */
/* BIOS INT10h is used to drive the display, and includes routines:         */
/*                                                                          */
/* 		set_direct_mode()   set 640x480 (2Eh) direct color mode			    */
/*      set_row(y)			set video memory page (segment) for horiz. line */
/*		clear_screen()		clear video memory with color black				*/
/*		putpixel()			set single pixel color (3 bytes) in RGB[]       */
/*		set_text_mode()		set display to 80 x 25 text mode
/*																			*/

#include <dos.h>
#include <math.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

/*  GLOBALS  */
char dac;
unsigned char far *vmdptr = 0xA0000000L;
unsigned int offset;
unsigned int page;
unsigned char RGB[3];

void set_direct_mode ()
/*set direct 24 bit color mode */
{
  union REGS regs;

/* Check DAC type installed */
  regs.h.ah = 0x10;
  regs.h.al = 0xF1;
  int86(0x10, &regs, &regs);
  dac = regs.h.bl;

  /*set 24 bit direct color mode for 640 x 480 mode 2Eh */
  regs.x.ax = 0x10f0;
  regs.h.bl = 0x2E;
  regs.h.ah = 0x10;
  regs.h.al = 0xE0;
  int86 (0x10, &regs, &regs);

  /*check for direct mode set- exit if not */
  if (regs.h.ah != 0)
	{
		printf("Direct Color mode not set\n");
		exit(1);
	}
}

void set_text_mode ()
/*set standard 80 x 25 text mode */
{
  union REGS regs;

  regs.x.ax = 0x03;
  regs.x.bx = 0x0;
  regs.x.cx = 0;
  regs.x.dx = 0;

  int86 (0x10, &regs, &regs);
}

void putpixel ()
/*set pixel Red Green Blue colors ( 3 bytes per pixel) */
{
  unsigned int i;
	/*loop on RGB colors */
	for(i=0; i<3; i++){
		vmdptr[offset++] = RGB[i];
		/*check for page (64Kb) boundary */
		if((offset & 0xFFFF) == 0){
			page++;
			outp(0x3CD, page);
		}
	}
}

void set_row(y)
int y;
/*determine (64Kb) page (segment) number to start row (2048 bytes per row)*/
{
   long x;

	x = ((long)y)*2048;
	page = x >> 16;
	offset = (unsigned int)x;
	outp(0x3CD, page);
}

void clear_screen()
/*set 2048 bytes to zero (black) in each line */
{
  unsigned int i, j;
  /* loop on rows */
  for(j=0; j<480; j++){
		set_row(j);
		/*loop on pixels in row*/
		for(i=0; i<2048; i++){
			vmdptr[offset++] = 0;     /* set to color black */
			/*check for page (64Kb) boundary */
			if((offset & 0xFFFF) == 0){
				page++;
				outp(0x3CD, page);
			}
		}
	}
}



main()
{
int  i, j;
int  d1, d2, d3;

/* set direct video mode 2Eh */
set_direct_mode();

clear_screen();

/*loop on rows */
for ( j=0; j<480; j+= 1 )
	{
	set_row(j);
	/*loop on pixels in line */
	for ( i=0; i<640; i+=1 )
		{
		/*Red origin upper left corner position */
		d1 = (int)sqrt( (double)i*(double)i + (double)j*(double)j );
		/*Green origin upper right corner position */
		d2 = (int)sqrt( (double)(i-640)*(double)(i-640) + (double)j*(double)j );
		/*Blue origin lower center position */
		d3 = (int)sqrt( (double)(j-480)*(double)(j-480) + (double)(i-320)*(double)(i-320) );

		/* set Red Green Blue colors- one byte each */
		RGB[0] = d1;   /* Red   */
		RGB[1] = d2;   /* Green */
		RGB[2] = d3;   /* Blue  */

		/*place the pixel on screen at j, i color RGB[] */
		putpixel();
		}
	}

/*display finished- beep and wait for any key press */
sound(450);
delay(100);
nosound();
getch();
set_text_mode();
printf("Wasn't that neat?\n");
printf("Installed DAC type = %x\n",dac);
getch();
}

